package com.ruoyi.system.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.entity.CommonEntity;
import com.ruoyi.system.entity.DragTable;
import com.ruoyi.system.entity.DragTableCondition;
import com.ruoyi.system.entity.TableSql;
import com.ruoyi.system.entity.vo.DragTableVo;
import com.ruoyi.system.mapper.CommonMapper;
import com.ruoyi.system.mapper.DragTableConditionMapper;
import com.ruoyi.system.mapper.TableSqlMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.DragTableMapper;
import com.ruoyi.system.service.IDragTableService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 动态格Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-07-31
 */
@Service
public class DragTableServiceImpl implements IDragTableService 
{
    @Autowired
    private DragTableMapper dragTableMapper;

    @Resource
    private DragTableConditionMapper dragTableConditionMapper;

    @Resource
    private TableSqlMapper tableSqlMapper;

    @Resource
    private CommonMapper commonMapper;

    /**
     * 查询动态格
     * 
     * @param tId 动态格主键
     * @return 动态格
     */
    @Override
    public DragTable selectDragTableByTId(Long tId)
    {
        return dragTableMapper.selectDragTableByTId(tId);
    }

    /**
     * 查询动态格列表
     * 
     * @param dragTable 动态格
     * @return 动态格
     */
    @Override
    public List<DragTable> selectDragTableList(DragTable dragTable)
    {
        return dragTableMapper.selectDragTableList(dragTable);
    }

    /**
     * 新增动态格
     * 
     * @param dragTable 动态格
     * @return 结果
     */
    @Override
    public int insertDragTable(DragTable dragTable)
    {
        dragTable.setCreateTime(DateUtils.getNowDate());
        return dragTableMapper.insertDragTable(dragTable);
    }

    /**
     * 修改动态格
     * 
     * @param dragTable 动态格
     * @return 结果
     */
    @Override
    public int updateDragTable(DragTable dragTable)
    {
        dragTable.setUpdateTime(DateUtils.getNowDate());
        return dragTableMapper.updateDragTable(dragTable);
    }

    /**
     * 批量删除动态格
     * 
     * @param tIds 需要删除的动态格主键
     * @return 结果
     */
    @Override
    public int deleteDragTableByTIds(Long[] tIds)
    {
        return dragTableMapper.deleteDragTableByTIds(tIds);
    }

    /**
     * 删除动态格信息
     * 
     * @param tId 动态格主键
     * @return 结果
     */
    @Override
    public int deleteDragTableByTId(Long tId)
    {
        return dragTableMapper.deleteDragTableByTId(tId);
    }

    @Override
    public CommonEntity dragTableInfo(String sqlKey) {
        DragTable dragTable = dragTableMapper.selectDragTableBySqlKey(sqlKey);
        Map<String,Object> resultMap = new HashMap<>();
        resultMap.put("template",dragTable);
//        resultMap.put("where",dragTableConditionMapper.selectDragTableConditionByTid(dragTable.gettId()));

        List<DragTableCondition> dragTableConditions = dragTableConditionMapper.selectDragTableConditionByTid(dragTable.gettId());
        dragTableConditions.forEach(d -> {
            // 0 查询字典 1 查询表数据
            if(d.getConditionDatasource() != null && d.getConditionDatasource().equals("0")){
                d.setDropDownList(commonMapper.selectDropDownListByDict(d.getConditionTableName()));
            }else if(d.getConditionDatasource() != null && d.getConditionDatasource().equals("1")){
                d.setDropDownList(commonMapper.selectDropDownList(d.getConditionTableName(),d.getConditionTableFieldKey(),d.getConditionTableFieldName(),new HashMap<>()));
            }
        });
        resultMap.put("where",dragTableConditions);
        resultMap.put("querySql",tableSqlMapper.selectTableSqlByTSqlKey(sqlKey));
        CommonEntity commonEntity = new CommonEntity();
        commonEntity.setResultMap(resultMap);
        return commonEntity;
    }

    @Transactional
    @Override
    public int addDragTable(DragTableVo dragTableVo) {

        //生成dtNickname表格别名
        String dtNickname = IdUtils.fastSimpleUUID();
        //生成sqlKey唯一标识
        String sqlKey = IdUtils.fastSimpleUUID();

        //add drag_table
        DragTable dragTable = new DragTable();
        BeanUtils.copyProperties(dragTableVo,dragTable);
        dragTable.setDtTableName(dtNickname);
        dragTable.setSqlKey(sqlKey);
        dragTable.setDtColumnName(dragTableVo.getDtColumnName().toString());

        //add table_sql
        TableSql tableSql = new TableSql();
        tableSql.setTableSql(dragTableVo.getTableSql());
        tableSql.setTableCondition(dragTableVo.getSearchFieldList().stream().collect(Collectors.joining(",")));
        tableSql.setTableAlias(dragTableVo.getDtTableName());
        tableSql.setSqlKey(sqlKey);
        tableSql.setTableExportField(dragTableVo.getTableExportField().toString());
        tableSqlMapper.insertTableSql(tableSql);

        //add drag_table_condition

        return dragTableMapper.insertDragTable(dragTable);
    }
}
